@@ -1,3 +1,4 @@
+from functools import partial
 import json
 import logging
 import sys
@@ -5,12 +6,13 @@ import sys
 import click
 
 import fiona
+from fiona.transform import transform_geom
 from fiona.fio.cli import cli
 
 
 def make_ld_context(context_items):
-    """Returns a JSON-LD Context object. 
-    
+    """Returns a JSON-LD Context object.
+
     See http://json-ld.org/spec/latest/json-ld."""
     ctx = {
       "@context": {
@@ -57,76 +59,28 @@ def id_record(rec):
     return rec
 
 
-def round_rec(rec, precision=None):
-    """Round coordinates of a geometric object to given precision."""
-    if precision is None:
-        return rec
-    geom = rec['geometry']
-    if geom['type'] == 'Point':
-        x, y = geom['coordinates']
-        xp, yp = [x], [y]
-        if precision is not None:
-            xp = [round(v, precision) for v in xp]
-            yp = [round(v, precision) for v in yp]
-        new_coords = tuple(zip(xp, yp))[0]
-    if geom['type'] in ['LineString', 'MultiPoint']:
-        xp, yp = zip(*geom['coordinates'])
-        if precision is not None:
-            xp = [round(v, precision) for v in xp]
-            yp = [round(v, precision) for v in yp]
-        new_coords = tuple(zip(xp, yp))
-    elif geom['type'] in ['Polygon', 'MultiLineString']:
-        new_coords = []
-        for piece in geom['coordinates']:
-            xp, yp = zip(*piece)
-            if precision is not None:
-                xp = [round(v, precision) for v in xp]
-                yp = [round(v, precision) for v in yp]
-            new_coords.append(tuple(zip(xp, yp)))
-    elif geom['type'] == 'MultiPolygon':
-        parts = geom['coordinates']
-        new_coords = []
-        for part in parts:
-            inner_coords = []
-            for ring in part:
-                xp, yp = zip(*ring)
-                if precision is not None:
-                    xp = [round(v, precision) for v in xp]
-                    yp = [round(v, precision) for v in yp]
-                inner_coords.append(tuple(zip(xp, yp)))
-            new_coords.append(inner_coords)
-    rec['geometry'] = {'type': geom['type'], 'coordinates': new_coords}
-    return rec
-
-
 # Cat command
 @cli.command(short_help="Concatenate and print the features of datasets")
-
 # One or more files.
 @click.argument('input', nargs=-1, type=click.Path(exists=True))
-
 # Coordinate precision option.
 @click.option('--precision', type=int, default=-1,
               help="Decimal precision of coordinates.")
-
-@click.option('--indent', default=None, type=int, 
+@click.option('--indent', default=None, type=int,
               help="Indentation level for pretty printed output.")
-
 @click.option('--compact/--no-compact', default=False,
               help="Use compact separators (',', ':').")
-
 @click.option('--ignore-errors/--no-ignore-errors', default=False,
               help="log errors but do not stop serialization.")
-
+@click.option('--dst_crs', default=None, help="Destination CRS.")
 # Use ASCII RS control code to signal a sequence item (False is default).
 # See http://tools.ietf.org/html/draft-ietf-json-text-sequence-05.
 # Experimental.
 @click.option('--x-json-seq-rs/--x-json-seq-no-rs', default=True,
         help="Use RS as text separator instead of LF. Experimental.")
-
 @click.pass_context
-
-def cat(ctx, input, precision, indent, compact, ignore_errors, x_json_seq_rs):
+def cat(ctx, input, precision, indent, compact, ignore_errors, dst_crs,
+        x_json_seq_rs):
     """Concatenate and print the features of input datasets as a
     sequence of GeoJSON features."""
     verbosity = ctx.obj['verbosity']
@@ -138,7 +92,6 @@ def cat(ctx, input, precision, indent, compact, ignore_errors, x_json_seq_rs):
         dump_kwds['indent'] = indent
     if compact:
         dump_kwds['separators'] = (',', ':')
-
     item_sep = compact and ',' or ', '
 
     try:
@@ -146,8 +99,12 @@ def cat(ctx, input, precision, indent, compact, ignore_errors, x_json_seq_rs):
             for path in input:
                 with fiona.open(path) as src:
                     for feat in src:
-                        if precision >= 0:
-                            feat = round_rec(feat, precision)
+                        if dst_crs or precision > 0:
+                            g = transform_geom(
+                                    src.crs, dst_crs, feat['geometry'],
+                                    antimeridian_cutting=True,
+                                    precision=precision)
+                            feat['geometry'] = g
                         if x_json_seq_rs:
                             sink.write(u'\u001e')
                         json.dump(feat, sink, **dump_kwds)
@@ -160,34 +117,26 @@ def cat(ctx, input, precision, indent, compact, ignore_errors, x_json_seq_rs):
 
 # Collect command
 @cli.command(short_help="Collect a sequence of features.")
-
 # Coordinate precision option.
 @click.option('--precision', type=int, default=-1,
               help="Decimal precision of coordinates.")
-
-@click.option('--indent', default=None, type=int, 
+@click.option('--indent', default=None, type=int,
               help="Indentation level for pretty printed output.")
-
 @click.option('--compact/--no-compact', default=False,
               help="Use compact separators (',', ':').")
-
 @click.option('--record-buffered/--no-record-buffered', default=False,
     help="Economical buffering of writes at record, not collection "
          "(default), level.")
-
 @click.option('--ignore-errors/--no-ignore-errors', default=False,
               help="log errors but do not stop serialization.")
-
+@click.option('--src_crs', default=None, help="Source CRS.")
 @click.option('--with-ld-context/--without-ld-context', default=False,
         help="add a JSON-LD context to JSON output.")
-
 @click.option('--add-ld-context-item', multiple=True,
         help="map a term to a URI and add it to the output's JSON LD context.")
-
 @click.pass_context
-
 def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
-            with_ld_context, add_ld_context_item):
+            src_crs, with_ld_context, add_ld_context_item):
     """Make a GeoJSON feature collection from a sequence of GeoJSON
     features and print it."""
     verbosity = ctx.obj['verbosity']
@@ -200,9 +149,14 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
         dump_kwds['indent'] = indent
     if compact:
         dump_kwds['separators'] = (',', ':')
-
     item_sep = compact and ',' or ', '
 
+    if src_crs:
+        transformer = partial(transform_geom, src_crs, 'EPSG:4326',
+                              antimeridian_cutting=True, precision=precision)
+    else:
+        transformer = lambda x: x
+
     first_line = next(stdin)
 
     # If input is RS-delimited JSON sequence.
@@ -212,17 +166,23 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
             for line in stdin:
                 if line.startswith(u'\x1e'):
                     if buffer:
-                        yield json.loads(buffer)
+                        feat = json.loads(buffer)
+                        feat['geometry'] = transformer(feat['geometry'])
+                        yield feat
                     buffer = line.strip(u'\x1e')
                 else:
                     buffer += line
             else:
-                yield json.loads(buffer)
+                feat = json.loads(buffer)
+                feat['geometry'] = transformer(feat['geometry'])
+                yield feat
     else:
         def feature_gen():
             yield json.loads(first_line)
             for line in stdin:
-                yield json.loads(line)
+                feat = json.loads(line)
+                feat['geometry'] = transformer(feat['geometry'])
+                yield feat
 
     try:
         source = feature_gen()
@@ -234,17 +194,17 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
             rec_indent = "\n" + " " * (2 * (indent or 0))
 
             collection = {
-                'type': 'FeatureCollection',  
+                'type': 'FeatureCollection',
                 'features': [] }
             if with_ld_context:
                 collection['@context'] = make_ld_context(
                     add_ld_context_item)
-            
+
             head, tail = json.dumps(collection, **dump_kwds).split('[]')
-            
+
             sink.write(head)
             sink.write("[")
-            
+
             # Try the first record.
             try:
                 i, first = 0, next(source)
@@ -252,8 +212,6 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
                     first = id_record(first)
                 if indented:
                     sink.write(rec_indent)
-                if precision >= 0:
-                    first = round_rec(first, precision)
                 sink.write(
                     json.dumps(first, **dump_kwds
                         ).replace("\n", rec_indent))
@@ -278,13 +236,11 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
                     if indented:
                         sink.write("\n")
                     raise
-            
+
             # Because trailing commas aren't valid in JSON arrays
             # we'll write the item separator before each of the
             # remaining features.
             for i, rec in enumerate(source, 1):
-                if precision >= 0:
-                    rec = round_rec(rec, precision)
                 try:
                     if with_ld_context:
                         rec = id_record(rec)
@@ -310,7 +266,7 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
                         if indented:
                             sink.write("\n")
                         raise
-            
+
             # Close up the GeoJSON after writing all features.
             sink.write("]")
             sink.write(tail)
@@ -323,19 +279,10 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
             if with_ld_context:
                 collection['@context'] = make_ld_context(
                     add_ld_context_item)
-                if precision >= 0:
-                    collection['features'] = [
-                        id_record(round_rec(rec, precision))
-                        for rec in source]
-                else:
-                    collection['features'] = [
-                        id_record(rec) for rec in source]
+                collection['features'] = [
+                    id_record(rec) for rec in source]
             else:
-                if precision >= 0:
-                    collection['features'] = [
-                        round_rec(rec, precision) for rec in source]
-                else:
-                    collection['features'] = list(source)
+                collection['features'] = list(source)
             json.dump(collection, sink, **dump_kwds)
             sink.write("\n")
 
@@ -347,46 +294,34 @@ def collect(ctx, precision, indent, compact, record_buffered, ignore_errors,
 
 # Dump command
 @cli.command(short_help="Dump a dataset to GeoJSON.")
-
 @click.argument('input', type=click.Path(), required=True)
-
 @click.option('--encoding', help="Specify encoding of the input file.")
-
 # Coordinate precision option.
 @click.option('--precision', type=int, default=-1,
               help="Decimal precision of coordinates.")
-
-@click.option('--indent', default=None, type=int, 
+@click.option('--indent', default=None, type=int,
               help="Indentation level for pretty printed output.")
-
 @click.option('--compact/--no-compact', default=False,
               help="Use compact separators (',', ':').")
-
 @click.option('--record-buffered/--no-record-buffered', default=False,
     help="Economical buffering of writes at record, not collection "
          "(default), level.")
-
 @click.option('--ignore-errors/--no-ignore-errors', default=False,
               help="log errors but do not stop serialization.")
-
 @click.option('--with-ld-context/--without-ld-context', default=False,
         help="add a JSON-LD context to JSON output.")
 
 @click.option('--add-ld-context-item', multiple=True,
         help="map a term to a URI and add it to the output's JSON LD context.")
-
 @click.option('--x-json-seq/--x-json-obj', default=False,
     help="Write a LF-delimited JSON sequence (default is object). "
          "Experimental.")
-
 # Use ASCII RS control code to signal a sequence item (False is default).
 # See http://tools.ietf.org/html/draft-ietf-json-text-sequence-05.
 # Experimental.
 @click.option('--x-json-seq-rs/--x-json-seq-no-rs', default=True,
         help="Use RS as text separator. Experimental.")
-
 @click.pass_context
-
 def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
          ignore_errors, with_ld_context, add_ld_context_item,
          x_json_seq, x_json_seq_rs):
@@ -401,13 +336,18 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
         dump_kwds['indent'] = indent
     if compact:
         dump_kwds['separators'] = (',', ':')
-
     item_sep = compact and ',' or ', '
 
     open_kwds = {}
     if encoding:
         open_kwds['encoding'] = encoding
 
+    def transformer(feat):
+        tg = partial(transform_geom, src_crs, 'EPSG:4326',
+                     antimeridian_cutting=True, precision=precision)
+        feat['geometry'] = tg(feat['geometry'])
+        return feat
+
     try:
         with fiona.drivers(CPL_DEBUG=verbosity>2):
             with fiona.open(input, **open_kwds) as source:
@@ -416,8 +356,7 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
 
                 if x_json_seq:
                     for feat in source:
-                        if precision >= 0:
-                            feat = round_rec(feat, precision)
+                        feat = transformer(feat)
                         if x_json_seq_rs:
                             sink.write(u'\u001e')
                         json.dump(feat, sink, **dump_kwds)
@@ -430,30 +369,29 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
                     rec_indent = "\n" + " " * (2 * (indent or 0))
 
                     collection = {
-                        'type': 'FeatureCollection',  
-                        'fiona:schema': meta['schema'], 
+                        'type': 'FeatureCollection',
+                        'fiona:schema': meta['schema'],
                         'fiona:crs': meta['crs'],
                         'features': [] }
                     if with_ld_context:
                         collection['@context'] = make_ld_context(
                             add_ld_context_item)
-                    
+
                     head, tail = json.dumps(collection, **dump_kwds).split('[]')
-                    
+
                     sink.write(head)
                     sink.write("[")
-                    
+
                     itr = iter(source)
-                    
+
                     # Try the first record.
                     try:
                         i, first = 0, next(itr)
+                        first = transformer(first)
                         if with_ld_context:
                             first = id_record(first)
                         if indented:
                             sink.write(rec_indent)
-                        if precision >= 0:
-                            first = round_rec(first, precision)
                         sink.write(
                             json.dumps(first, **dump_kwds
                                 ).replace("\n", rec_indent))
@@ -478,13 +416,12 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
                             if indented:
                                 sink.write("\n")
                             raise
-                    
+
                     # Because trailing commas aren't valid in JSON arrays
                     # we'll write the item separator before each of the
                     # remaining features.
                     for i, rec in enumerate(itr, 1):
-                        if precision >= 0:
-                            rec = round_rec(rec, precision)
+                        rec = transformer(rec)
                         try:
                             if with_ld_context:
                                 rec = id_record(rec)
@@ -510,7 +447,7 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
                                 if indented:
                                     sink.write("\n")
                                 raise
-                    
+
                     # Close up the GeoJSON after writing all features.
                     sink.write("]")
                     sink.write(tail)
@@ -520,31 +457,19 @@ def dump(ctx, input, encoding, precision, indent, compact, record_buffered,
                 else:
                     # Buffer GeoJSON data at the collection level. The default.
                     collection = {
-                        'type': 'FeatureCollection', 
-                        'fiona:schema': meta['schema'], 
+                        'type': 'FeatureCollection',
+                        'fiona:schema': meta['schema'],
                         'fiona:crs': meta['crs']}
                     if with_ld_context:
                         collection['@context'] = make_ld_context(
                             add_ld_context_item)
-                        if precision >= 0:
-                            collection['features'] = [
-                                id_record(round_rec(rec, precision))
-                                for rec in source]
-                        else:
-                            collection['features'] = [
-                                id_record(rec) for rec in source]
+                        collection['features'] = [
+                            id_record(transformer(rec)) for rec in source]
                     else:
-                        if precision >= 0:
-                            collection['features'] = [
-                                round_rec(rec, precision) for rec in source]
-                        else:
-                            collection['features'] = list(source)
+                        collection['features'] = [transformer(rec) for rec in source]
                     json.dump(collection, sink, **dump_kwds)
 
         sys.exit(0)
     except Exception:
         logger.exception("Failed. Exception caught")
         sys.exit(1)
-
-
-
